#define STRICT
#include <windows.h>
#include <commctrl.h>
#include "filters.h" 
#include "resource.h"
#include "BladeMP3EncDLL.h"

#ifndef _DEBUG
#define TINY_DLL
#endif

#ifdef TINY_DLL

#define malloc(x) LocalAlloc(LMEM_FIXED,x)
#define free(x) LocalFree(x)

char * t_strdup(const char * x)
{
	char * r=(char*)malloc(sizeof(char)*(strlen(x)+1));
	strcpy(r,x);
	return r;
}

#define strdup t_strdup

#endif

BEINITSTREAM beInitStream;
BEENCODECHUNK beEncodeChunk;
BEDEINITSTREAM beDeinitStream;
BECLOSESTREAM beCloseStream;
BEWRITEINFOTAG beWriteInfoTag;

static HINSTANCE hLame;

int lame_init()
{
	if (hLame) return 1;
	hLame=LoadLibrary("lame_enc.dll");
	if (!hLame) return 0;
	beInitStream=(BEINITSTREAM)GetProcAddress(hLame,"beInitStream");
	if (!beInitStream) goto fail;
	beEncodeChunk=(BEENCODECHUNK)GetProcAddress(hLame,"beEncodeChunk");
	if (!beEncodeChunk) goto fail;
	beDeinitStream=(BEDEINITSTREAM)GetProcAddress(hLame,"beDeinitStream");
	if (!beDeinitStream) goto fail;
	beCloseStream=(BECLOSESTREAM)GetProcAddress(hLame,"beCloseStream");
	if (!beCloseStream) goto fail;
	beWriteInfoTag=(BEWRITEINFOTAG)GetProcAddress(hLame,"beWriteInfoTag");
	if (beWriteInfoTag) return 1;
fail:
	FreeLibrary(hLame);
	hLame=0;
	return 0;
}

void lame_kill()
{
	if (hLame)
	{
		FreeLibrary(hLame);
		hLame=0;
	}
}


static const struct
{
	LAME_QUALITY_PRESET id;
	const char * name;
} presets[]=
{
	{LQP_STANDARD,"standard"},
	{LQP_FAST_STANDARD,"fast standard"},
	{LQP_EXTREME,"extreme"},
	{LQP_FAST_EXTREME,"fast extreme"},
	{LQP_INSANE,"insane"},
	{LQP_ABR,"ABR"},
	{LQP_CBR,"CBR"},
	{LQP_MEDIUM,"medium"},
	{LQP_FAST_MEDIUM,"fast medium"},
};

#define N_PRESETS (sizeof(presets)/sizeof(presets[0]))


static WORD bitrates[]={32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320};
#define N_BITRATES (sizeof(bitrates)/sizeof(bitrates[0]))

void PASCAL GetSuggestedSampleType(long* sr,short *bps,short *ch)
{
	*ch=2;
	*sr=44100;
	*bps=16;
}

short FAR PASCAL QueryCoolFilter(COOLQUERY far * cq)
{   
	strcpy(cq->szName,"MPEG layer-3 (LAME)");
	strcpy(cq->szCopyright,"http://www.mp3dev.org/");
	strcpy(cq->szExt,"MP3");
	cq->lChunkSize=0x1000;
	cq->dwFlags=QF_CANSAVE|QF_HASOPTIONSBOX;
 	cq->Stereo8=0;
 	cq->Stereo16=R_32075|R_44100|R_48000;
 	cq->Stereo24=0;
 	cq->Stereo32=0;
 	cq->Mono8=0;
 	cq->Mono16=R_32075|R_44100|R_48000;
 	cq->Mono24=0;
 	cq->Mono32=0;
 	cq->Quad32=0;
 	cq->Quad16=0;
 	cq->Quad8=0;
 	return C_VALIDLIBRARY;
}

BOOL WINAPI DlgProc(HWND wnd,UINT msg,WPARAM wp,LPARAM lp)
{
	int br, part1, part2;
	switch(msg)
	{
	case WM_INITDIALOG:
		{
			HWND w;
			UINT n;
			int set;
			long idx;
			w=GetDlgItem(wnd,IDC_PRESET);
			set=0;
			for(n=0;n<N_PRESETS;n++)
			{
				idx=SendMessage(w,CB_ADDSTRING,0,(long)presets[n].name);
				if (presets[n].id==(lp&0xFFFF)) {set=1;SendMessage(w,CB_SETCURSEL,idx,0);}
			}
			if (!set) SendMessage(w,CB_SETCURSEL,0,0);
			w=GetDlgItem(wnd,IDC_BITRATE);
			set=0;
			for(n=0;n<N_BITRATES;n++)
			{
				char tmp[16];
				wsprintf(tmp,"%u",bitrates[n]);
				idx=SendMessage(w,CB_ADDSTRING,0,(long)tmp);
				if (bitrates[n]==(lp>>16)) SendMessage(w,CB_SETCURSEL,idx,0);
			}
			if (!set) SendMessage(w,CB_SETCURSEL,9,0);
		}
		break;                       // return removed and break added by John Edwards 2002-09-14
	case WM_COMMAND:                     // This section modified by John Edwards 2002-09-14
		switch (LOWORD(wp))
		{
			case IDCANCEL:
				EndDialog(wnd,0);
				break;
			case IDOK:
				part1 = SendDlgItemMessage(wnd,IDC_BITRATE,CB_GETCURSEL,
					(WPARAM) 0, (LPARAM) 0);

				if ((br = SendDlgItemMessage(wnd,IDC_PRESET,CB_GETCURSEL,
						(WPARAM) 0, (LPARAM) 0)) != CB_ERR)

					part2 = SendDlgItemMessage(wnd,IDC_PRESET,CB_GETCURSEL,
							(WPARAM) br, (LPARAM) 0) + 6;

				EndDialog(wnd, ((DWORD)bitrates[part1]<<16) | part2);
				break;
		}                            // End of section modified by John Edwards 2002-09-14
	}
	return 0;
}


DWORD PASCAL FilterGetOptions(HWND hWnd, HINSTANCE hInst, long lSamprate, WORD wChannels, WORD wBitsPerSample, DWORD cfg)
{
	DWORD r=DialogBoxParam(hInst,(char*)IDD_DIALOG1,hWnd,DlgProc,cfg);
	return r ? r : cfg;
}


DWORD FAR PASCAL FilterWriteSpecialData(HANDLE hOutput, LPCSTR szListType, LPCSTR szType, char * pData,DWORD dwSize)
{
	return 0;
}

typedef struct
{
	HANDLE f;
	HBE_STREAM str;
	void* src;
	DWORD sz;
	char* name;
	DWORD os;
	BYTE *buf;
} OUTPUT;

OUTPUT* PASCAL OpenFilterOutput(LPSTR fn,long sr,WORD bps,WORD nch,long size, long *chunk, DWORD config)
{
	OUTPUT* out;
	HANDLE f;
	HBE_STREAM str;
	BE_CONFIG bcfg;
	DWORD os,ns;

	if (!lame_init())
	{
		return 0;
	}
	
	out=0;


	
	f=CreateFile(fn,GENERIC_WRITE,0,0,CREATE_ALWAYS,0,0);

	if (f==INVALID_HANDLE_VALUE) goto err;
	ZeroMemory(&bcfg,sizeof(BE_CONFIG));
	bcfg.dwConfig=BE_CONFIG_LAME;
	bcfg.format.LHV1.dwStructVersion=1;
	bcfg.format.LHV1.dwStructSize=sizeof(bcfg.format.LHV1);
	bcfg.format.LHV1.dwSampleRate=sr;
	bcfg.format.LHV1.nMode= (nch == 2) ? BE_MP3_MODE_JSTEREO : BE_MP3_MODE_MONO;
                                      // Modified to JOINT-STEREO by John Edwards 2002-09-14
		
	bcfg.format.LHV1.nPreset=config&0xFFFF;

	switch (bcfg.format.LHV1.nPreset)  // This section modified by John Edwards 2002-09-14 & 2004-02-09 for medium
	{
		case LQP_STANDARD:
		case LQP_FAST_STANDARD:
		case LQP_EXTREME:
		case LQP_FAST_EXTREME:
		case LQP_MEDIUM:
		case LQP_FAST_MEDIUM:
			bcfg.format.LHV1.bWriteVBRHeader=1;
			bcfg.format.LHV1.bEnableVBR=1;
			bcfg.format.LHV1.dwBitrate=0;
//			bcfg.format.LHV1.dwMaxBitrate=320;
			break;
		case LQP_INSANE:
			bcfg.format.LHV1.bEnableVBR=0;
			break;
		case LQP_ABR:
			bcfg.format.LHV1.bWriteVBRHeader=1;
			bcfg.format.LHV1.bEnableVBR=1;
			bcfg.format.LHV1.dwVbrAbr_bps=(config>>16) * 1000;
			break;
		case LQP_CBR:
			bcfg.format.LHV1.bEnableVBR=0;
			bcfg.format.LHV1.dwBitrate=config>>16;
			break;
	}                                   // End of section modified by John Edwards 2002-09-14

	if (beInitStream(&bcfg, &ns, &os, &str)) {CloseHandle(f);goto err;}

	out=(OUTPUT*)malloc(sizeof(OUTPUT));
	out->buf=(BYTE*)malloc(os);
	out->name=strdup(fn);
	out->str=str;
	out->f=f;
	out->os=os;

	*chunk=ns*2;
	return out;
err:
	if (out) free(out);
	return 0;
}


void FAR PASCAL CloseFilterOutput(OUTPUT* out)
{
	DWORD os,bw;
	beDeinitStream(out->str, out->buf,&os);
	WriteFile(out->f,out->buf,os,&bw,0);
	CloseHandle(out->f);
	beWriteInfoTag(out->str,out->name);
	beCloseStream(out->str);
	free(out->name);
	free(out->buf);
	free(out);
}

DWORD FAR PASCAL WriteFilterOutput(OUTPUT* out, unsigned char far *buf, long lBytes)
{
	DWORD os,bw;
	out->src=buf;
	out->sz=lBytes/2;

	beEncodeChunk(out->str, out->sz, (short*)out->src, out->buf, &os);
	WriteFile(out->f,out->buf,os,&bw,0);

	return lBytes;
}

#ifdef TINY_DLL
int _stdcall _DllMainCRTStartup(int a,int b,int c) {return 1;}
#endif
